@@ -25,6 +25,18 @@ http://media.example.com/entire2.ts
 #EXT-X-ENDLIST
 '''
 
+SIMPLE_PLAYLIST_WITH_VERY_SHORT_DURATION = '''
+#EXTM3U
+#EXT-X-TARGETDURATION:5220
+#EXTINF:5220,
+http://media.example.com/entire1.ts
+#EXTINF:5218.5,
+http://media.example.com/entire2.ts
+#EXTINF:0.000011,
+http://media.example.com/entire3.ts
+#EXT-X-ENDLIST
+'''
+
 SIMPLE_PLAYLIST_WITH_START_NEGATIVE_OFFSET = '''
 #EXTM3U
 #EXT-X-TARGETDURATION:5220
@@ -141,6 +153,14 @@ http://example.com/hi.m3u8
 http://example.com/audio-only.m3u8
 '''
 
+VARIANT_PLAYLIST_WITH_VIDEO_RANGE = '''
+#EXTM3U
+#EXT-X-STREAM-INF:PROGRAM-ID=1,VIDEO-RANGE=SDR"
+http://example.com/sdr.m3u8
+#EXT-X-STREAM-INF:PROGRAM-ID=1,VIDEO-RANGE=PQ"
+http://example.com/hdr.m3u8
+'''
+
 VARIANT_PLAYLIST_WITH_BANDWIDTH_FLOAT = '''
 #EXTM3U
 #EXT-X-STREAM-INF:PROGRAM-ID=1, BANDWIDTH=1280000.0
@@ -877,11 +897,22 @@ fileSequence272.ts
 #EXT-X-PART:DURATION=0.33334,URI="filePart273.1.ts"
 #EXT-X-PART:DURATION=0.33334,URI="filePart273.2.ts"
 #EXT-X-PART:DURATION=0.33334,URI="filePart273.3.ts"
+#EXT-X-PRELOAD-HINT:TYPE=PART,URI="filePart273.4.ts"
 
 #EXT-X-RENDITION-REPORT:URI="../1M/waitForMSN.php",LAST-MSN=273,LAST-PART=3
 #EXT-X-RENDITION-REPORT:URI="../4M/waitForMSN.php",LAST-MSN=273,LAST-PART=3
 '''
 
+LOW_LATENCY_WITH_PRELOAD_AND_BYTERANGES_PLAYLIST = '''
+#EXTM3U
+#EXTINF:4.08,
+fs270.mp4
+#EXT-X-PART:DURATION=1.02,URI="fs271.mp4",BYTERANGE=20000@0
+#EXT-X-PART:DURATION=1.02,URI="fs271.mp4",BYTERANGE=23000@20000
+#EXT-X-PART:DURATION=1.02,URI="fs271.mp4",BYTERANGE=18000@43000
+#EXT-X-PRELOAD-HINT:TYPE=PART,URI="fs271.mp4",BYTERANGE-START=61000,BYTERANGE-LENGTH=20000
+'''
+
 RELATIVE_PLAYLIST_FILENAME = abspath(join(dirname(__file__), 'playlists/relative-playlist.m3u8'))
 
 RELATIVE_PLAYLIST_URI = TEST_HOST + '/path/to/relative-playlist.m3u8'
@@ -902,6 +933,18 @@ http://example.com/hi.m3u8
 http://example.com/audio-only.m3u8
 '''
 
+VARIANT_PLAYLIST_WITH_ROUNDABLE_FRAME_RATE = '''
+#EXTM3U
+#EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=65000,FRAME-RATE=12.54321,CODECS="mp4a.40.5,avc1.42801e"
+http://example.com/audio-only.m3u8
+'''
+
+VARIANT_PLAYLIST_WITH_ROUNDED_FRAME_RATE = '''
+#EXTM3U
+#EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=65000,FRAME-RATE=12.543,CODECS="mp4a.40.5,avc1.42801e"
+http://example.com/audio-only.m3u8
+'''
+
 SESSION_DATA_PLAYLIST = '''#EXTM3U
 #EXT-X-VERSION:4
 #EXT-X-SESSION-DATA:DATA-ID="com.example.value",VALUE="example",LANGUAGE="en"
@@ -928,4 +971,93 @@ http://media.example.com/entire.ts
 #EXT-X-ENDLIST
 '''
 
+DATERANGE_SIMPLE_PLAYLIST = '''
+#EXTM3U
+#EXT-X-PROGRAM-DATE-TIME:2016-06-13T11:15:15Z
+#EXT-X-DATERANGE:ID="ad3",START-DATE="2016-06-13T11:15:00Z",DURATION=20,X-AD-URL="http://ads.example.com/beacon3",X-AD-ID="1234"
+#EXTINF:10,
+ad3.1.ts
+#EXTINF:10,
+ad3.2.ts
+'''
+
+DATERANGE_SCTE35_OUT_AND_IN_PLAYLIST = '''
+#EXTM3U
+# adapted from https://tools.ietf.org/html/rfc8216#section-8.10
+#EXT-X-PROGRAM-DATE-TIME:2014-03-05T11:15:00Z
+#EXT-X-DATERANGE:ID="splice-6FFFFFF0",START-DATE="2014-03-05T11:15:00Z",PLANNED-DURATION=59.993,SCTE35-OUT=0xFC002F0000000000FF000014056FFFFFF000E011622DCAFF000052636200000000000A0008029896F50000008700000000
+#EXTINF:10,
+ad3.1.ts
+#EXTINF:10,
+ad3.2.ts
+#EXTINF:10,
+ad3.3.ts
+#EXTINF:10,
+ad3.4.ts
+#EXTINF:10,
+ad3.5.ts
+#EXTINF:10,
+ad3.6.ts
+#EXT-X-DATERANGE:ID="splice-6FFFFFF0",DURATION=59.993,SCTE35-IN=0xFC002A0000000000FF00000F056FFFFFF000401162802E6100000000000A0008029896F50000008700000000
+#EXTINF:10,
+prog.1.ts
+'''
+
+DATERANGE_ENDDATE_SCTECMD_PLAYLIST = '''
+#EXTM3U
+#EXT-X-PROGRAM-DATE-TIME:2020-03-11T10:51:00Z
+#EXT-X-DATERANGE:ID="test_id",START-DATE="2020-03-11T10:51:00Z",CLASS="test_class",END-DATE="2020-03-11T10:52:00Z",DURATION=60,SCTE35-CMD=0xFCINVALIDSECTION
+#EXTINF:10,
+prog.1.ts
+'''
+
+DATERANGE_IN_PART_PLAYLIST = '''
+#EXTM3U
+#EXT-X-PROGRAM-DATE-TIME:2020-03-10T07:48:00Z
+#EXT-X-PART:DURATION=1,URI="filePart271.a.ts"
+#EXT-X-PART:DURATION=1,URI="filePart271.b.ts"
+#EXT-X-DATERANGE:ID="test_id",START-DATE="2020-03-10T07:48:02Z",CLASS="test_class",END-ON-NEXT=YES
+#EXT-X-PART:DURATION=1,URI="filePart271.c.ts"
+'''
+
+GAP_PLAYLIST = '''
+#EXTM3U
+#EXT-X-MEDIA-SEQUENCE:14
+#EXT-X-VERSION:7
+#EXT-X-TARGETDURATION:10
+#EXTINF:9.84317,
+fileSequence14.ts
+#EXTINF:8.75875,
+#EXT-X-GAP
+missing-Sequence15.ts
+#EXTINF:9.88487,
+#EXT-X-GAP
+missing-Sequence16.ts
+#EXTINF:9.09242,
+fileSequence17.ts
+'''
+
+GAP_IN_PARTS_PLAYLIST = '''
+#EXTM3U
+#EXT-X-PART:DURATION=1,URI="filePart271.a.ts"
+#EXT-X-PART:DURATION=1,URI="filePart271.b.ts",GAP=YES
+#EXT-X-GAP
+#EXT-X-PART:DURATION=1,URI="filePart271.c.ts"
+'''
+
+PLAYLIST_WITH_SLASH_IN_QUERY_STRING = '''
+#EXTM3U
+#EXT-X-VERSION:3
+#EXT-X-TARGETDURATION:5
+#EXT-X-MEDIA-SEQUENCE:10599
+#EXT-X-PROGRAM-DATE-TIME:2020-08-05T13:51:49.000+00:00
+#EXTINF:5.0000,
+testvideo-1596635509-4769390994-a0e3087c.ts?hdntl=exp=1596678764~acl=/*~data=hdntl~hmac=12345&
+#EXTINF:5.0000,
+testvideo-1596635514-4769840994-a0e00878.ts?hdntl=exp=1596678764~acl=/*~data=hdntl~hmac=12345&
+#EXTINF:5.0000,
+testvideo-1596635519-4770290994-a0e5087d.ts?hdntl=exp=1596678764~acl=/*~data=hdntl~hmac=12345&
+#EXTINF:5.0000,
+'''
+
 del abspath, dirname, join
