@@ -11,11 +11,11 @@ import os
 import pytest
 import sys
 
-from m3u8.protocol import ext_x_start, ext_x_part
+from m3u8.protocol import ext_x_start, ext_x_part, ext_x_preload_hint
 
 import m3u8
 import playlists
-from m3u8.model import Segment, Key, Media, MediaList, RenditionReport, PartialSegment, denormalize_attribute, find_key, SessionData
+from m3u8.model import Segment, Key, Media, MediaList, RenditionReport, PartialSegment, denormalize_attribute, find_key, SessionData, PreloadHint, DateRange
 
 
 class UTC(datetime.tzinfo):
@@ -35,6 +35,13 @@ class UTC(datetime.tzinfo):
 utc = UTC()
 
 
+def test_base_path_playlist_with_slash_in_query_string():
+    playlist = m3u8.M3U8(
+        playlists.PLAYLIST_WITH_SLASH_IN_QUERY_STRING,
+        base_path='http://testvideo.com/foo'
+    )
+    assert playlist.segments[0].uri == 'http://testvideo.com/foo/testvideo-1596635509-4769390994-a0e3087c.ts?hdntl=exp=1596678764~acl=/*~data=hdntl~hmac=12345&'
+
 def test_target_duration_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     mock_parser_data(obj, {'targetduration': '1234567'})
@@ -645,6 +652,14 @@ def test_dump_should_not_ignore_zero_duration():
     assert "EXTINF:5220" in obj.dumps().strip()
 
 
+def test_dump_should_use_decimal_floating_point_for_very_short_durations():
+    obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST_WITH_VERY_SHORT_DURATION)
+
+    assert "EXTINF:5220" in obj.dumps().strip()
+    assert "EXTINF:5218.5" in obj.dumps().strip()
+    assert "EXTINF:0.000011" in obj.dumps().strip()
+
+
 def test_dump_should_include_segment_level_program_date_time():
     obj = m3u8.M3U8(playlists.DISCONTINUITY_PLAYLIST_WITH_PROGRAM_DATE_TIME)
     # Tag being expected is in the segment level, not the global one
@@ -1039,6 +1054,12 @@ def test_should_dump_frame_rate():
 
     assert expected == obj.dumps().strip()
 
+def test_should_round_frame_rate():
+    obj = m3u8.M3U8(playlists.VARIANT_PLAYLIST_WITH_ROUNDABLE_FRAME_RATE)
+    expected = playlists.VARIANT_PLAYLIST_WITH_ROUNDED_FRAME_RATE.strip()
+
+    assert expected == obj.dumps().strip()
+
 @pytest.mark.skipif(sys.version_info >= (3,), reason="unicode not available in v3")
 def test_m3u8_unicode_method():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
@@ -1089,25 +1110,31 @@ def test_find_key_throws_when_no_match():
 
 def test_ll_playlist():
     obj = m3u8.M3U8(playlists.LOW_LATENCY_DELTA_UPDATE_PLAYLIST)
+    obj.base_path = 'http://localhost/test_base_path'
+    obj.base_uri = 'http://localhost/test_base_uri'
+
     assert len(obj.rendition_reports) == 2
     assert len(obj.segments[2].parts) == 12
-    print(obj.dumps())
-    assert (ext_x_part + ':DURATION=0.33334,URI="filePart271.0.ts"') in obj.dumps()
+    assert (ext_x_part + ':DURATION=0.33334,URI="http://localhost/test_base_path/filePart271.0.ts"') in obj.dumps()
+    assert (ext_x_preload_hint + ':TYPE=PART,URI="http://localhost/test_base_path/filePart273.4.ts"') in obj.dumps()
+    assert obj.preload_hint.base_uri == 'http://localhost/test_base_uri'
 
 def test_add_rendition_report_to_playlist():
     obj = m3u8.M3U8()
 
     obj.add_rendition_report(
         RenditionReport(
-            base_uri='',
+            base_uri=None,
             uri='../1M/waitForMSN.php',
             last_msn=273,
             last_part=0
         )
     )
 
+    obj.base_path = 'http://localhost/test'
+
     result = obj.dumps()
-    expected = '#EXT-X-RENDITION-REPORT:URI="../1M/waitForMSN.php",LAST-MSN=273,LAST-PART=0'
+    expected = '#EXT-X-RENDITION-REPORT:URI="http://localhost/test/waitForMSN.php",LAST-MSN=273,LAST-PART=0'
 
     assert expected in result
 
@@ -1119,7 +1146,7 @@ def test_add_part_to_segment():
 
     obj.add_part(
         PartialSegment(
-            '',
+            None,
             'filePart271.0.ts',
             0.33334
         )
@@ -1188,6 +1215,129 @@ def test_endswith_newline():
 
     assert manifest.endswith('#EXT-X-ENDLIST\n')
 
+def test_init_section_base_path_update():
+    obj = m3u8.M3U8(playlists.MULTIPLE_MAP_URI_PLAYLIST)
+
+    assert obj.segments[0].init_section.uri == 'init1.mp4'
+
+    obj.base_path = 'http://localhost/base_path'
+    obj.base_uri = 'http://localhost/base_uri'
+
+    assert obj.segments[0].init_section.uri == 'http://localhost/base_path/init1.mp4'
+    assert obj.segments[0].init_section.base_uri == 'http://localhost/base_uri'
+
+def test_iframe_playlists_base_path_update():
+    obj = m3u8.M3U8(playlists.VARIANT_PLAYLIST_WITH_IFRAME_PLAYLISTS)
+
+    assert obj.iframe_playlists[0].uri == 'video-800k-iframes.m3u8'
+    assert obj.iframe_playlists[0].base_uri == None
+
+    obj.base_path = 'http://localhost/base_path'
+    obj.base_uri = 'http://localhost/base_uri'
+
+    assert obj.iframe_playlists[0].uri == 'http://localhost/base_path/video-800k-iframes.m3u8'
+    assert obj.iframe_playlists[0].base_uri == 'http://localhost/base_uri'
+
+def test_partial_segment_base_path_update():
+    obj = m3u8.M3U8(playlists.LOW_LATENCY_DELTA_UPDATE_PLAYLIST)
+
+    obj.base_path = 'http://localhost/base_path'
+    obj.base_uri = 'http://localhost/base_uri'
+
+    assert obj.segments[2].parts[0].uri == 'http://localhost/base_path/filePart271.0.ts'
+    assert obj.segments[2].parts[0].base_uri == 'http://localhost/base_uri'
+
+def test_add_preload_hint():
+    obj = PreloadHint(
+        'PART',
+        '',
+        'filePart273.4.ts',
+        0
+    )
+
+    result = obj.dumps()
+    expected = '#EXT-X-PRELOAD-HINT:TYPE=PART,URI="filePart273.4.ts",BYTERANGE-START=0'
+
+    assert result == expected
+
+def test_add_daterange():
+    obj = DateRange(
+        id='testid123',
+        start_date='2020-03-09T17:19:00Z',
+        planned_duration=60,
+        x_test_client_attr='"test-attr"'
+    )
+
+    result = obj.dumps()
+    expected = '#EXT-X-DATERANGE:ID="testid123",START-DATE="2020-03-09T17:19:00Z",PLANNED-DURATION=60,X-TEST-CLIENT-ATTR="test-attr"'
+
+    assert result == expected
+
+def test_daterange_simple():
+    obj = m3u8.M3U8(playlists.DATERANGE_SIMPLE_PLAYLIST)
+
+    # note that x-<client-attribute>s are explicitly alphabetically ordered
+    # when dumped for predictability, so line below is different from input
+    expected = '#EXT-X-DATERANGE:ID="ad3",START-DATE="2016-06-13T11:15:00Z",DURATION=20,X-AD-ID="1234",X-AD-URL="http://ads.example.com/beacon3"'
+    result = obj.dumps()
+
+    assert expected in result
+
+def test_daterange_scte_out_and_in():
+    obj = m3u8.M3U8(playlists.DATERANGE_SCTE35_OUT_AND_IN_PLAYLIST)
+
+    result = obj.dumps()
+
+    daterange_out = '#EXT-X-DATERANGE:ID="splice-6FFFFFF0",START-DATE="2014-03-05T11:15:00Z",PLANNED-DURATION=59.993,SCTE35-OUT=0xFC002F0000000000FF000014056FFFFFF000E011622DCAFF000052636200000000000A0008029896F50000008700000000'
+    daterange_in = '#EXT-X-DATERANGE:ID="splice-6FFFFFF0",DURATION=59.993,SCTE35-IN=0xFC002A0000000000FF00000F056FFFFFF000401162802E6100000000000A0008029896F50000008700000000'
+
+    assert daterange_out in result
+    assert daterange_in in result
+
+def test_daterange_enddate_sctecmd():
+    obj = m3u8.M3U8(playlists.DATERANGE_ENDDATE_SCTECMD_PLAYLIST)
+
+    result = obj.dumps()
+    expected = '#EXT-X-DATERANGE:ID="test_id",START-DATE="2020-03-11T10:51:00Z",CLASS="test_class",END-DATE="2020-03-11T10:52:00Z",DURATION=60,SCTE35-CMD=0xFCINVALIDSECTION'
+
+    assert expected in result
+
+def test_daterange_in_parts():
+    obj = m3u8.M3U8(playlists.DATERANGE_IN_PART_PLAYLIST)
+
+    result = obj.dumps()
+    expected = '#EXT-X-DATERANGE:ID="test_id",START-DATE="2020-03-10T07:48:02Z",CLASS="test_class",END-ON-NEXT=YES'
+
+    assert expected in result
+
+def test_add_gap():
+    obj = m3u8.Segment(
+        uri='fileSequence271.ts',
+        duration=4,
+        gap_tag=True
+    )
+
+    result = str(obj)
+    expected = '#EXTINF:4,\n#EXT-X-GAP\nfileSequence271.ts'
+
+    assert result == expected
+
+def test_gap():
+    obj = m3u8.M3U8(playlists.GAP_PLAYLIST)
+
+    result = obj.dumps().strip()
+    expected = playlists.GAP_PLAYLIST.strip()
+
+    assert result == expected
+
+def test_gap_in_parts():
+    obj = m3u8.M3U8(playlists.GAP_IN_PARTS_PLAYLIST)
+
+    result = obj.dumps().strip()
+    expected = playlists.GAP_IN_PARTS_PLAYLIST.strip()
+
+    assert result == expected
+
 # custom asserts
 
 
