@@ -173,6 +173,12 @@ def test_should_parse_variant_playlist_with_average_bandwidth():
     assert 65000 == playlists_list[3]['stream_info']['bandwidth']
     assert 63005 == playlists_list[3]['stream_info']['average_bandwidth']
 
+def test_should_parse_variant_playlist_with_video_range():
+    data = m3u8.parse(playlists.VARIANT_PLAYLIST_WITH_VIDEO_RANGE)
+    playlists_list = list(data['playlists'])
+    assert 'SDR' == playlists_list[0]['stream_info']['video_range']
+    assert 'PQ' == playlists_list[1]['stream_info']['video_range']
+
 # This is actually not according to specification but as for example Twitch.tv
 # is producing master playlists that have bandwidth as floats (issue 72)
 # this tests that this situation does not break the parser and will just
@@ -336,6 +342,14 @@ def test_should_parse_empty_uri_with_base_path():
     assert 'base_uri/' == media.base_uri
 
 
+def test_should_parse_audio_channels():
+    data = m3u8.M3U8(
+            playlists.MEDIA_WITHOUT_URI_PLAYLIST,
+            base_path='base_path', base_uri='base_uri')
+    media = data.media[0]
+    assert media.channels == "2"
+
+
 def test_should_parse_start_with_negative_time_offset():
     data = m3u8.parse(playlists.SIMPLE_PLAYLIST_WITH_START_NEGATIVE_OFFSET)
     assert data['start']['time_offset'] == -2.0
@@ -400,6 +414,11 @@ def test_master_playlist_with_frame_rate():
     assert 60 == playlists_list[2]['stream_info']['frame_rate']
     assert 12.5 == playlists_list[3]['stream_info']['frame_rate']
 
+def test_master_playlist_with_unrounded_frame_rate():
+    data = m3u8.parse(playlists.VARIANT_PLAYLIST_WITH_ROUNDABLE_FRAME_RATE)
+    playlists_list = list(data['playlists'])
+    assert 12.54321 == playlists_list[0]['stream_info']['frame_rate']
+
 def test_low_latency_playlist():
     data = m3u8.parse(playlists.LOW_LATENCY_DELTA_UPDATE_PLAYLIST)
     assert data['server_control']['can_block_reload'] == 'YES'
@@ -415,7 +434,60 @@ def test_low_latency_playlist():
     assert data['rendition_reports'][0]['last_msn'] == 273
     assert data['rendition_reports'][0]['last_part'] == 3
 
+def test_low_latency_with_preload_and_byteranges_playlist():
+    data = m3u8.parse(playlists.LOW_LATENCY_WITH_PRELOAD_AND_BYTERANGES_PLAYLIST)
+    assert data['segments'][1]['parts'][2]['byterange'] == "18000@43000"
+    assert data['preload_hint']['type'] == 'PART'
+    assert data['preload_hint']['uri'] == 'fs271.mp4'
+    assert data['preload_hint']['byterange_start'] == 61000
+    assert data['preload_hint']['byterange_length'] == 20000
 
 def test_negative_media_sequence():
     data = m3u8.parse(playlists.PLAYLIST_WITH_NEGATIVE_MEDIA_SEQUENCE)
     assert data['media_sequence'] == -2680
+
+def test_daterange_simple():
+    data = m3u8.parse(playlists.DATERANGE_SIMPLE_PLAYLIST)
+
+    assert data['segments'][0]['dateranges'][0]['id'] == 'ad3'
+    assert data['segments'][0]['dateranges'][0]['start_date'] == "2016-06-13T11:15:00Z"
+    assert data['segments'][0]['dateranges'][0]['duration'] == 20
+    assert data['segments'][0]['dateranges'][0]['x_ad_id'] == '"1234"'
+    assert data['segments'][0]['dateranges'][0]['x_ad_url'] == '"http://ads.example.com/beacon3"'
+
+def test_date_range_with_scte_out_and_in():
+    data = m3u8.parse(playlists.DATERANGE_SCTE35_OUT_AND_IN_PLAYLIST)
+
+    assert data['segments'][0]['dateranges'][0]['id'] == 'splice-6FFFFFF0'
+    assert data['segments'][0]['dateranges'][0]['planned_duration'] == 59.993
+    assert data['segments'][0]['dateranges'][0]['scte35_out'] =='0xFC002F0000000000FF000014056FFFFFF000E011622DCAFF000052636200000000000A0008029896F50000008700000000'
+
+    assert data['segments'][6]['dateranges'][0]['id'] == 'splice-6FFFFFF0'
+    assert data['segments'][6]['dateranges'][0]['duration'] == 59.993
+    assert data['segments'][6]['dateranges'][0]['scte35_in'] == '0xFC002A0000000000FF00000F056FFFFFF000401162802E6100000000000A0008029896F50000008700000000'
+
+def test_date_range_in_parts():
+    data = m3u8.parse(playlists.DATERANGE_IN_PART_PLAYLIST)
+
+    assert data['segments'][0]['parts'][2]['dateranges'][0]['id'] == 'test_id'
+    assert data['segments'][0]['parts'][2]['dateranges'][0]['start_date'] == '2020-03-10T07:48:02Z'
+    assert data['segments'][0]['parts'][2]['dateranges'][0]['class'] == 'test_class'
+    assert data['segments'][0]['parts'][2]['dateranges'][0]['end_on_next'] == 'YES'
+
+def test_gap():
+    data = m3u8.parse(playlists.GAP_PLAYLIST)
+
+    assert data['segments'][0]['gap_tag'] is None
+    assert data['segments'][1]['gap_tag'] == True
+    assert data['segments'][2]['gap_tag'] == True
+    assert data['segments'][3]['gap_tag'] is None
+
+def test_gap_in_parts():
+    data = m3u8.parse(playlists.GAP_IN_PARTS_PLAYLIST)
+
+    assert data['segments'][0]['parts'][0]['gap_tag'] is None
+    assert data['segments'][0]['parts'][0].get('gap', None) is None
+    assert data['segments'][0]['parts'][1]['gap_tag'] is None
+    assert data['segments'][0]['parts'][1]['gap'] == 'YES'
+    assert data['segments'][0]['parts'][2]['gap_tag'] == True
+    assert data['segments'][0]['parts'][2].get('gap', None) is None
